@@ -110,6 +110,9 @@ class PetscConfig:
     def __getitem__(self, item):
         return self.configdict[item]
 
+    def get(self, item, default=None):
+        return self.configdict.get(item, default)
+
     def configure(self, extension, compiler=None):
         self.configure_extension(extension)
         if compiler is not None:
@@ -183,7 +186,9 @@ class PetscConfig:
         petsc_inc = flaglist(self['PETSC_CC_INCLUDES'])
         petsc_lib = flaglist(
             '-L%s %s' % (self['PETSC_LIB_DIR'], self['PETSC_LIB_BASIC']))
-        petsc_lib['runtime_library_dirs'].append(self['PETSC_LIB_DIR'])
+        # runtime_library_dirs is not supported on Windows
+        if sys.platform != 'win32':
+            petsc_lib['runtime_library_dirs'].append(self['PETSC_LIB_DIR'])
 
         # Link in extra libraries on static builds
         if self['BUILDSHAREDLIB'] != 'yes':
@@ -195,48 +200,39 @@ class PetscConfig:
 
     def configure_compiler(self, compiler):
         if compiler.compiler_type != 'unix': return
-        (cc, cxx, cflags, ccshared,
-         ldflags, ldshared, so_ext) = get_config_vars(
-            'CC', 'CXX', 'CFLAGS',  'CCSHARED',
-            'LDFLAGS', 'LDSHARED', 'SO')
-        cflags = cflags or ''
-        ldflags = ldflags or ''
+        getenv = os.environ.get
+        # distutils C/C++ compiler
+        (cc, cflags, ccshared, cxx) = get_config_vars(
+            'CC', 'CFLAGS',  'CCSHARED', 'CXX')
+        ccshared = getenv('CCSHARED', ccshared or '')
+        cflags = getenv('CFLAGS', cflags or '')
         cflags = cflags.replace('-Wstrict-prototypes', '')
+        # distutils linker
+        (ldflags, ldshared, so_ext) = get_config_vars(
+            'LDFLAGS', 'LDSHARED', 'SO')
         ld = cc
-        ldshared = ldshared.replace(ld, '', 1).strip()
-        ldshared = [flg for flg in split_quoted(ldshared)
-                    if flg not in split_quoted(ldflags)]
+        ldshared = getenv('LDSHARED', ldshared)
+        ldflags = getenv('LDFLAGS', cflags + ' ' + (ldflags or ''))
+        ldcmd = split_quoted(ld) + split_quoted(ldflags)
+        ldshared = [flg for flg in split_quoted(ldshared) if flg not in ldcmd]
         ldshared = str.join(' ', ldshared)
         #
-        getenv = os.environ.get
         def get_flags(cmd):
             try: return ' '.join(split_quoted(cmd)[1:])
             except: return ''
-        # C compiler
+        # PETSc C compiler
         PCC = self['PCC']
         PCC_FLAGS = get_flags(cc) + ' ' + self['PCC_FLAGS']
         PCC_FLAGS = PCC_FLAGS.replace('-fvisibility=hidden', '')
-        if sys.version_info[:2] < (2, 5):
-            PCC_FLAGS = PCC_FLAGS.replace('-Wwrite-strings', '')
         PCC = getenv('PCC', PCC) + ' ' +  getenv('PCCFLAGS', PCC_FLAGS)
-        ccshared = getenv('CCSHARED', ccshared)
-        cflags   = getenv('CFLAGS',   cflags)
         PCC_SHARED = str.join(' ', (PCC, ccshared, cflags))
-        # C++ compiler
-        if self.language == 'c++':
-            PCXX = PCC
-        else:
-            try:
-                PCXX = self['CXX']
-            except KeyError:
-                PCXX = cxx
-        # linker
+        # PETSc C++ compiler
+        PCXX = PCC if self.language == 'c++' else self.get('CXX', cxx)
+        # PETSc linker
         PLD = self['PCC_LINKER']
         PLD_FLAGS = get_flags(ld) + ' ' + self['PCC_LINKER_FLAGS']
         PLD_FLAGS = PLD_FLAGS.replace('-fvisibility=hidden', '')
         PLD = getenv('PLD', PLD) + ' ' + getenv('PLDFLAGS', PLD_FLAGS)
-        ldshared = getenv('LDSHARED', ldshared)
-        ldflags  = getenv('LDFLAGS',  cflags + ' ' + ldflags)
         PLD_SHARED = str.join(' ', (PLD, ldshared, ldflags))
         #
         compiler.set_executables(
@@ -548,7 +544,7 @@ PETSC_ARCH = %(PETSC_ARCH)s
         for ext in self.extensions:
             fullname = self.get_ext_fullname(ext.name)
             filename = self.get_ext_filename(fullname)
-            if isinstance(ext, Extension):
+            if isinstance(ext, Extension) and self.petsc_arch:
                 head, tail = os.path.split(filename)
                 for arch in self.petsc_arch:
                     outfile = os.path.join(self.build_lib,
@@ -601,6 +597,23 @@ class sdist(_sdist):
 
 # --------------------------------------------------------------------
 
+if setuptools:
+    try:
+        from setuptools.command import egg_info as mod_egg_info
+        _FileList = mod_egg_info.FileList
+        class FileList(_FileList):
+            def process_template_line(self, line):
+                level = log.set_threshold(log.ERROR)
+                try:
+                    _FileList.process_template_line(self, line)
+                finally:
+                    log.set_threshold(level)
+        mod_egg_info.FileList = FileList
+    except:
+        pass
+
+# --------------------------------------------------------------------
+
 def append(seq, item):
     if item not in seq:
         seq.append(item)
